#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributers.

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.remoteDriver import remoteDriver
from fenrirscreenreader.core.eventData import fenrirEventType

import select, socket, os, os.path


class driver(remoteDriver):
    def __init__(self):
        remoteDriver.__init__(self)
    def initialize(self, environment):
        self.env = environment
        self.env['runtime']['processManager'].addCustomEventThread(self.watchDog, multiprocess=True)
    def watchDog(self, active, eventQueue):
        # echo "command say this is a test" | socat - UNIX-CLIENT:/tmp/fenrirscreenreader-deamon.sock
        socketFile = ''
        try:
            socketFile = self.env['runtime']['settingsManager'].getSetting('remote', 'socketFile')
        except:
            pass
        if socketFile == '':
            if self.env['runtime']['settingsManager'].getSetting('screen', 'driver') =='vcsaDriver':
                socketFile =  '/tmp/fenrirscreenreader-deamon.sock'
            else:
                socketFile = '/tmp/fenrirscreenreader-' + str(os.getppid()) + '.sock'
        if os.path.exists(socketFile):
            os.unlink(socketFile)
        self.fenrirSock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.fenrirSock.bind(socketFile)
        os.chmod(socketFile, 0o666)
        self.fenrirSock.listen(1)
        while active.value:
            # Check if the client is still connected and if data is available:
            try:
                r, _, _ = select.select([self.fenrirSock], [], [], 0.8)
            except select.error:
                break
            if r == []:
                continue
            if self.fenrirSock in r:
                client_sock, client_addr = self.fenrirSock.accept()
            response = "ERROR: Failed to process command\n"
            try:
                rawdata = client_sock.recv(8129)
                data = rawdata.decode("utf-8").rstrip().lstrip()
                if data:
                    # Process the command and get response
                    result = self.env['runtime']['remoteManager'].handleRemoteIncommingWithResponse(data)
                    if result['success']:
                        response = f"OK: {result['message']}\n"
                    else:
                        response = f"ERROR: {result['message']}\n"
                else:
                    response = "ERROR: Empty command\n"
            except Exception as e:
                response = f"ERROR: {str(e)}\n"
            
            # Send response back to client
            try:
                client_sock.send(response.encode("utf-8"))
            except:
                pass
            
            try:
                client_sock.close()
            except:
                pass
        if self.fenrirSock:
            self.fenrirSock.close()
            self.fenrirSock = None
        if os.path.exists(socketFile):
            os.unlink(socketFile)
